package cmdVM

import (
	"fmt"
	"reflect"
	"strings"

	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/common/vm"
	nc "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/nexusclient"
	"gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/libclient/utils"
	u "gitedu.hesge.ch/flg_projects/nexus_vdi/nexus/nexush/utils"
)

type List struct {
	Name string
}

func (cmd *List) GetName() string {
	return cmd.Name
}

func (cmd *List) GetDesc() []string {
	return []string{
		"Lists VMs you own or on which you can perform an action."}
}

func (cmd *List) PrintUsage() {
	for _, desc := range cmd.GetDesc() {
		u.PrintlnErr(desc)
	}
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("USAGE: ", cmd.GetName(), " [-l] [ID ...] [regex ...]")
	u.PrintlnErr("―――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――――")
	u.PrintlnErr("Use \"-l\" to specify detailed VMs output.")
	printRegexUsageDetails()
}

func (cmd *List) Run(client *nc.NexusClient, args []string) int {
	return cmd.printFilteredVMs(client, args)
}

// Prints a list of filtered VMs for a given route.
// Return 0 if everything went well or 1 in case of failure.
func (cmd *List) printFilteredVMs(client *nc.NexusClient, args []string) int {
	if len(args) < 1 {
		cmd.PrintUsage()
		return 1
	}

	// Check if a "-l" argument is specified
	foundLongOutputFlag := -1
	for idx, arg := range args {
		if arg == "-l" {
			foundLongOutputFlag = idx
		}
	}

	if foundLongOutputFlag >= 0 {
		args = utils.RemoveArgAtIndex(args, foundLongOutputFlag)
	}

	vms, err := getFilteredVMs(client, args)
	if err != nil {
		u.PrintlnErr("Error: " + err.Error())
		return 1
	}
	if len(vms) < 1 {
		u.PrintlnErr("VM(s) not found")
		return 1
	}

	statusCode := 0

	if foundLongOutputFlag >= 0 {
		for _, vm := range vms {
			str, err := vm.String()
			if err != nil {
				u.PrintlnErr("Failed decoding VM " + vm.ID.String() + " to string. Skipped.")
				statusCode = 1
			} else {
				u.Println(str)
			}
		}
	} else {
		// Compute the length of the longest name and state (used for padding columns)
		nameLen := 0
		idLen := len(vms[0].ID.String()) // ID is a UUID of constant length
		stateLen := 0
		for _, vm := range vms {
			l := len(vm.Name)
			if l > nameLen {
				nameLen = l
			}
			l = len(vm.State)
			if l > stateLen {
				stateLen = l
			}
		}

		paddedFmt := fmt.Sprintf("%%-%ds | %%-%ds | %%-%ds |", nameLen, idLen, stateLen)

		for _, vm := range vms {
			fmt.Printf(paddedFmt, vm.Name, vm.ID, vm.State)
			actions := cmd.vmActionsToString(client, &vm)
			fmt.Println(actions)
		}
	}

	return statusCode
}

func (cmd *List) vmActionsToString(client *nc.NexusClient, v *vm.VMNetworkSerialized) string {
	// Use reflection to convert the caps.VMActions structure
	// into a string (based on the fields names)
	actions := client.GetVMActions(v)
	val := reflect.ValueOf(actions)
	typ := reflect.TypeOf(actions)

	var builder strings.Builder

	for i := range val.NumField() {
		field := typ.Field(i)
		fieldValue := val.Field(i)
		value := fieldValue.Interface().(bool)
		if value {
			builder.WriteString(" ")
			builder.WriteString(strings.ToLower(field.Name))
		}
	}

	return builder.String()
}
